import {
  select,
  json,
  tsv,
  geoPath,
  geoNaturalEarth1,
  zoom,
  event
} from 'd3';
import { feature } from 'topojson';

const svg = select('svg');
const margin = {
  top: 70,
  bottom: 0,
  left: 10, 
  right: 10
};
const width = document.body.clientWidth;
const innerWidth = width - (margin.left + margin.right);
const height = document.body.clientHeight;
const innerHeight = height - (margin.top + margin.bottom);

const projection = geoNaturalEarth1();
const pathGenerator = geoPath().projection(projection);
let tsvData = [];
let topoJSONdata = [];
let countryNames = [];
let selectedCountries = [];

const g = svg.append('g')
  .attr('transform', `translate(${margin.left}, ${margin.top}) scale(${(innerWidth/width)}, ${(innerHeight/height)})`);

const countryText = g.append('text')
  .attr('class', 'title')
  .attr('x', innerWidth / 2)
  .attr('y', -5)
  .text('');

g.append('path')
    .attr('class', 'sphere')
    .attr('d', pathGenerator({type: 'Sphere'}));

svg.call(zoom().on('zoom', () => {
  g.attr('transform', event.transform);
}));

Promise.all([
  tsv('https://unpkg.com/world-atlas@1.1.4/world/50m.tsv'),
  json('https://unpkg.com/world-atlas@1.1.4/world/50m.json')
]).then(([tsvD, topoD]) => {
  tsvData = tsvD;
  topoJSONdata = topoD
  
  countryNames = tsvData.reduce((accumulator, d) => {
    accumulator[d.iso_n3] = d.name;
    return accumulator;
  }, {});
  //console.log(countryNames);
  render();  
});

const selectCountry = c => {
  //console.log(c);
  if(!selectedCountries.includes(c.id))
    selectedCountries.push(c.id);
  else {
    let idx = selectedCountries.indexOf(c.id);
    if (idx > -1)
      selectedCountries.splice(idx, 1);
  }
  console.log(c.id);
  render();
};

const setTitleText = str => {
  countryText.text(str);
};

const render = () => {  
  const countries = feature(topoJSONdata, topoJSONdata.objects.countries);
  const countryPaths = g.selectAll('.country').data(countries.features);
  countryPaths
    .enter().append('path')
      .attr('class', 'country')
      .attr('d', pathGenerator)
    .merge(countryPaths)
      .attr('fill', d => selectedCountries.includes(d.id)
          ? 'red' 
          : 'lightgreen')
      .on('click', d => selectCountry(d))
      .on('mouseover', d => setTitleText(countryNames[d.id]))
      .on('mouseout', d => setTitleText())
    .append('title')
      .text(d => countryNames[d.id])
};